/*
	Shallow Pointer Template Class

    This class allows a Large Object to be managed by an STL container

    Use:
        <list <ShallowPointer LargeObject> >    Manager ;

        This instantiates a list of LargeObjects called Manager.
*/

template <class T>
class ShallowPointer 
{
    public:
        ShallowPointer(T *pData);
        ShallowPointer(const ShallowPointer &rhs);
        ~ShallowPointer() throw() ;

        // De-reference this Pointer to return a pointer to the
        // Object of type T as a T *const. 
        T *const GetPointer() const;

        // De-reference this Pointer to return a pointer to the
        // Object of type T as a T *const. Thus providing
        // 'transparent' access to members of T from the List 
        // without adding them to this class.
        T *const operator ->() const;
        T & operator *() const;

        // Used by List::sort()
        int operator <(const ShallowPointer &rhs);

        // Used by List::
        int operator ==(const ShallowPointer &rhs);

    private:    // Hide from Compiler & Users
        ShallowPointer();
        const ShallowPointer<T> & operator=(const ShallowPointer<T> &right);

    private:    // Attributes
        // The pointer to our Object
        T *pPointer;
};


